@@ -2,7 +2,7 @@
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2021 Lance Edgar
+#  Copyright © 2010-2022 Lance Edgar
 #
 #  This file is part of Rattail.
 #
@@ -247,6 +247,7 @@ class JQueryAutocompleteWidget(dfwidget.AutocompleteInputWidget):
     template = 'autocomplete_jquery'
     requirements = None
     field_display = ""
+    assigned_label = None
     service_url = None
     cleared_callback = None
     selected_callback = None
@@ -275,6 +276,7 @@ class JQueryAutocompleteWidget(dfwidget.AutocompleteInputWidget):
         kw['options'] = json.dumps(options)
         kw['field_display'] = self.field_display
         kw['cleared_callback'] = self.cleared_callback
+        kw['assigned_label'] = self.assigned_label
         kw.setdefault('selected_callback', self.selected_callback)
         tmpl_values = self.get_template_values(field, cstruct, kw)
         template = readonly and self.readonly_template or self.template
