@@ -926,7 +926,8 @@ class AlchemyGPCFilter(AlchemyGridFilter):
     """
     GPC filter for SQLAlchemy.
     """
-    default_verbs = ['equal', 'not_equal', 'equal_any_of']
+    default_verbs = ['equal', 'not_equal', 'equal_any_of',
+                     'is_null', 'is_not_null']
 
     def filter_equal(self, query, value):
         """
