@@ -95,6 +95,22 @@ const TailboneAutocomplete = {
         }
     },
 
+    watch: {
+        // TODO: yikes this feels hacky.  what happens is, when the
+        // caller explicitly assigns a new UUID value to the tailbone
+        // autocomplate component, the underlying buefy autocomplete
+        // component was not getting the new value.  so here we are
+        // explicitly making sure it is in sync.  this issue was
+        // discovered on the "new vendor catalog batch" page
+        value(val) {
+            this.$nextTick(() => {
+                if (this.buefyValue != val) {
+                    this.buefyValue = val
+                }
+            })
+        },
+    },
+
     methods: {
 
         // fetch new search results from the server.  this is invoked
