@@ -2,7 +2,7 @@
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2021 Lance Edgar
+#  Copyright © 2010-2022 Lance Edgar
 #
 #  This file is part of Rattail.
 #
@@ -82,6 +82,7 @@ class BatchMasterView(MasterView):
     results_executable = False
     has_worksheet = False
     has_worksheet_file = False
+    delete_requires_progress = True
 
     input_file_template_config_section = 'rattail.batch'
 
@@ -686,6 +687,8 @@ class BatchMasterView(MasterView):
                 return HTML.tag('p', c=[link])
 
     def make_batch_row_grid_tools(self, batch):
+        if self.get_use_buefy():
+            return
         if self.rows_bulk_deletable and not batch.executed and self.request.has_perm('{}.delete_rows'.format(self.get_permission_prefix())):
             url = self.request.route_url('{}.delete_rows'.format(self.get_route_prefix()), uuid=batch.uuid)
             return HTML.tag('p', c=[tags.link_to("Delete all rows matching current search", url)])
@@ -740,8 +743,55 @@ class BatchMasterView(MasterView):
         """
         Delete all data (files etc.) for the batch.
         """
-        self.handler.do_delete(batch)
-        super(BatchMasterView, self).delete_instance(batch)
+        app = self.get_rattail_app()
+        session = app.get_session(batch)
+        self.batch_handler.do_delete(batch)
+        session.flush()
+
+    def delete_instance_with_progress(self, batch):
+        """
+        Delete all data (files etc.) for the batch.
+        """
+        return self.handler_action(batch, 'delete')
+
+    def delete_thread(self, key, user_uuid, progress, **kwargs):
+        """
+        Thread target for deleting a batch with progress indicator.
+        """
+        app = self.get_rattail_app()
+        model = self.model
+        # nb. must make new session, separate from main thread
+        session = app.make_session()
+        batch = self.get_instance_for_key(key, session)
+        batch_str = six.text_type(batch)
+
+        try:
+            # try to delete batch
+            self.handler.do_delete(batch, progress=progress, **kwargs)
+
+        except Exception as error:
+            # error; log that and rollback
+            log.exception("delete failed for batch: %s", batch)
+            session.rollback()
+            session.close()
+            if progress:
+                progress.session.load()
+                progress.session['error'] = True
+                progress.session['error_msg'] = "Batch deletion failed: {}".format(
+                    simple_error(error))
+                progress.session.save()
+
+        else:
+            # no error; finish up
+            session.commit()
+            session.close()
+            if progress:
+                progress.session.load()
+                progress.session['complete'] = True
+                progress.session['success_url'] = self.get_index_url()
+                progress.session['success_msg'] = "Batch has been deleted: {}".format(
+                    batch_str)
+                progress.session.save()
 
     def get_fallback_templates(self, template, **kwargs):
         return [
