@@ -2,7 +2,7 @@
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2021 Lance Edgar
+#  Copyright © 2010-2022 Lance Edgar
 #
 #  This file is part of Rattail.
 #
@@ -84,6 +84,11 @@ class LabelBatchView(BatchMasterView):
         'upc': "UPC",
         'vendor_id': "Vendor ID",
         'label_profile': "Label Type",
+        'sale_start': "Sale Starts",
+        'sale_stop': "Sale Ends",
+        'tpr_price': "TPR Price",
+        'tpr_starts': "TPR Starts",
+        'tpr_ends': "TPR Ends",
     }
 
     row_form_fields = [
@@ -101,6 +106,12 @@ class LabelBatchView(BatchMasterView):
         'sale_price',
         'sale_start',
         'sale_stop',
+        'tpr_price',
+        'tpr_starts',
+        'tpr_ends',
+        'current_price',
+        'current_starts',
+        'current_ends',
         'vendor_id',
         'vendor_name',
         'vendor_item_code',
