@@ -2,7 +2,7 @@
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2021 Lance Edgar
+#  Copyright © 2010-2022 Lance Edgar
 #
 #  This file is part of Rattail.
 #
@@ -30,16 +30,16 @@ import logging
 
 import six
 
-from rattail.db import model, api
-from rattail.vendors.catalogs import iter_catalog_parsers
+from rattail.db import model
 
 import colander
 from deform import widget as dfwidget
 from webhelpers2.html import tags
 
 from tailbone import forms
-from tailbone.db import Session
 from tailbone.views.batch import FileBatchMasterView
+from tailbone.diffs import Diff
+from tailbone.db import Session
 
 
 log = logging.getLogger(__name__)
@@ -55,11 +55,15 @@ class VendorCatalogView(FileBatchMasterView):
     route_prefix = 'vendorcatalogs'
     url_prefix = '/vendors/catalogs'
     template_prefix = '/batch/vendorcatalog'
-    editable = False
+    bulk_deletable = True
+    results_executable = True
     rows_bulk_deletable = True
+    has_input_file_templates = True
+    configurable = True
 
     labels = {
         'vendor_id': "Vendor ID",
+        'parser_key': "Parser",
     }
 
     grid_columns = [
@@ -74,11 +78,13 @@ class VendorCatalogView(FileBatchMasterView):
 
     form_fields = [
         'id',
-        'description',
-        'vendor',
         'filename',
+        'parser_key',
+        'vendor',
         'future',
         'effective',
+        'params',
+        'description',
         'notes',
         'created',
         'created_by',
@@ -112,16 +118,6 @@ class VendorCatalogView(FileBatchMasterView):
         'description',
         'size',
         'is_preferred_vendor',
-        'old_vendor_code',
-        'vendor_code',
-        'old_case_size',
-        'case_size',
-        'old_case_cost',
-        'case_cost',
-        'case_cost_diff',
-        'old_unit_cost',
-        'unit_cost',
-        'unit_cost_diff',
         'suggested_retail',
         'starts',
         'ends',
@@ -129,19 +125,25 @@ class VendorCatalogView(FileBatchMasterView):
         'discount_ends',
         'discount_amount',
         'discount_percent',
+        'case_cost_diff',
+        'unit_cost_diff',
         'status_code',
         'status_text',
     ]
 
+    def get_input_file_templates(self):
+        return [
+            {'key': 'default',
+             'label': "Default",
+             'default_url': self.request.static_url(
+                 'tailbone:static/files/vendor_catalog_template.xlsx')},
+        ]
+
     def get_parsers(self):
         if not hasattr(self, 'parsers'):
-            parsers = sorted(iter_catalog_parsers(), key=lambda p: p.display)
-            supported = self.rattail_config.getlist(
-                'tailbone', 'batch.vendorcatalog.supported_parsers')
-            if supported:
-                parsers = [parser for parser in parsers
-                           if parser.key in supported]
-            self.parsers = parsers
+            app = self.get_rattail_app()
+            vendor_handler = app.get_vendor_handler()
+            self.parsers = vendor_handler.get_supported_catalog_parsers()
         return self.parsers
 
     def configure_grid(self, g):
@@ -156,24 +158,8 @@ class VendorCatalogView(FileBatchMasterView):
 
     def configure_form(self, f):
         super(VendorCatalogView, self).configure_form(f)
-
-        # vendor
-        f.set_renderer('vendor', self.render_vendor)
-        if self.creating and 'vendor' in f:
-            f.replace('vendor', 'vendor_uuid')
-            f.set_node('vendor_uuid', colander.String())
-            vendor_display = ""
-            if self.request.method == 'POST':
-                if self.request.POST.get('vendor_uuid'):
-                    vendor = self.Session.query(model.Vendor).get(self.request.POST['vendor_uuid'])
-                    if vendor:
-                        vendor_display = six.text_type(vendor)
-            vendors_url = self.request.route_url('vendors.autocomplete')
-            f.set_widget('vendor_uuid', forms.widgets.JQueryAutocompleteWidget(
-                field_display=vendor_display, service_url=vendors_url))
-            f.set_label('vendor_uuid', "Vendor")
-        else:
-            f.set_readonly('vendor')
+        app = self.get_rattail_app()
+        vendor_handler = app.get_vendor_handler()
 
         # filename
         f.set_label('filename', "Catalog File")
@@ -190,13 +176,90 @@ class VendorCatalogView(FileBatchMasterView):
             if not use_buefy:
                 values.insert(0, ('', "(please choose)"))
             f.set_widget('parser_key', dfwidget.SelectWidget(values=values))
-            f.set_label('parser_key', "File Type")
-
-        # effective
-        if self.creating:
-            f.remove('effective')
         else:
-            f.set_readonly('effective')
+            f.set_readonly('parser_key')
+            f.set_renderer('parser_key', self.render_parser_key)
+
+        # vendor
+        f.set_renderer('vendor', self.render_vendor)
+        if self.creating and 'vendor' in f:
+            f.replace('vendor', 'vendor_uuid')
+            f.set_label('vendor_uuid', "Vendor")
+            use_dropdown = vendor_handler.choice_uses_dropdown()
+            if use_dropdown:
+                vendors = self.Session.query(model.Vendor)\
+                                      .order_by(model.Vendor.id)
+                vendor_values = [(vendor.uuid, "({}) {}".format(vendor.id,
+                                                                vendor.name))
+                                 for vendor in vendors]
+                f.set_widget('vendor_uuid',
+                             dfwidget.SelectWidget(values=vendor_values))
+            else:
+                vendor_display = ""
+                if self.request.method == 'POST':
+                    if self.request.POST.get('vendor_uuid'):
+                        vendor = self.Session.query(model.Vendor).get(
+                            self.request.POST['vendor_uuid'])
+                        if vendor:
+                            vendor_display = six.text_type(vendor)
+                vendors_url = self.request.route_url('vendors.autocomplete')
+                f.set_widget('vendor_uuid', forms.widgets.JQueryAutocompleteWidget(
+                    field_display=vendor_display, service_url=vendors_url,
+                    assigned_label='vendorName'))
+        else:
+            f.set_readonly('vendor')
+
+        if self.batch_handler.allow_future():
+
+            # effective
+            f.set_type('effective', 'date_jquery')
+
+        else: # future not allowed
+            f.remove('future',
+                     'effective')
+
+    def render_parser_key(self, batch, field):
+        key = getattr(batch, field)
+        if not key:
+            return
+        app = self.get_rattail_app()
+        vendor_handler = app.get_vendor_handler()
+        parser = vendor_handler.get_catalog_parser(key)
+        return parser.display
+
+    def template_kwargs_create(self, **kwargs):
+        use_buefy = self.get_use_buefy()
+        app = self.get_rattail_app()
+        vendor_handler = app.get_vendor_handler()
+        parsers = self.get_parsers()
+        parsers_data = {}
+        for parser in parsers:
+            if use_buefy:
+                pdata = {'key': parser.key,
+                         'vendor_key': parser.vendor_key}
+                if parser.vendor_key:
+                    vendor = vendor_handler.get_vendor(self.Session(),
+                                                       parser.vendor_key)
+                    if vendor:
+                        pdata['vendor_uuid'] = vendor.uuid
+                        pdata['vendor_name'] = vendor.name
+                parsers_data[parser.key] = pdata
+            else:
+                if parser.vendor_key:
+                    vendor = vendor_handler.get_vendor(self.Session(),
+                                                       parser.vendor_key)
+                    if vendor:
+                        parser.vendormap_value = "{{uuid: '{}', name: '{}'}}".format(
+                            vendor.uuid, vendor.name.replace("'", "\\'"))
+                    else:
+                        log.warning("vendor '{}' not found for parser: {}".format(
+                            parser.vendor_key, parser.key))
+                        parser.vendormap_value = 'null'
+                else:
+                    parser.vendormap_value = 'null'
+        kwargs['parsers'] = parsers
+        kwargs['parsers_data'] = parsers_data
+        return kwargs
 
     def get_batch_kwargs(self, batch):
         kwargs = super(VendorCatalogView, self).get_batch_kwargs(batch)
@@ -209,7 +272,9 @@ class VendorCatalogView(FileBatchMasterView):
             kwargs['vendor_id'] = batch.vendor_id
         if batch.vendor_name:
             kwargs['vendor_name'] = batch.vendor_name
-        kwargs['future'] = batch.future
+        if self.batch_handler.allow_future():
+            kwargs['future'] = batch.future
+            kwargs['effective'] = batch.effective
         return kwargs
 
     def configure_row_grid(self, g):
@@ -218,7 +283,7 @@ class VendorCatalogView(FileBatchMasterView):
 
         # starts
         if not batch.future:
-            g.hide_column('starts')
+            g.remove('starts')
 
         g.set_type('old_unit_cost', 'currency')
         g.set_type('unit_cost', 'currency')
@@ -252,27 +317,77 @@ class VendorCatalogView(FileBatchMasterView):
         f.set_type('upc', 'gpc')
         f.set_type('discount_percent', 'percent')
 
-    def template_kwargs_index(self, **kwargs):
-        url = self.rattail_config.get('tailbone', 'batch.vendorcatalog.generic_template_url')
-        kwargs['generic_template_url'] = url
+    def template_kwargs_view_row(self, **kwargs):
+        row = kwargs['instance']
+        batch = row.batch
+
+        fields = [
+            'vendor_code',
+            'case_size',
+            'case_cost',
+            'unit_cost',
+        ]
+        old_data = dict([(field, getattr(row, 'old_{}'.format(field)))
+                         for field in fields])
+        new_data = dict([(field, getattr(row, field))
+                         for field in fields])
+        kwargs['catalog_entry_diff'] = Diff(old_data, new_data, fields=fields,
+                                            monospace=True)
+
         return kwargs
 
-    def template_kwargs_create(self, **kwargs):
-        parsers = self.get_parsers()
-        for parser in parsers:
-            if parser.vendor_key:
-                vendor = api.get_vendor(Session(), parser.vendor_key)
-                if vendor:
-                    parser.vendormap_value = "{{uuid: '{}', name: '{}'}}".format(
-                        vendor.uuid, vendor.name.replace("'", "\\'"))
-                else:
-                    log.warning("vendor '{}' not found for parser: {}".format(
-                        parser.vendor_key, parser.key))
-                    parser.vendormap_value = 'null'
-            else:
-                parser.vendormap_value = 'null'
-        kwargs['parsers'] = parsers
-        return kwargs
+    def configure_get_simple_settings(self):
+        settings = super(VendorCatalogView, self).configure_get_simple_settings() or []
+        settings.extend([
+
+            # key field
+            {'section': 'rattail.batch',
+             'option': 'vendor_catalog.allow_future',
+             'type': bool},
+
+        ])
+        return settings
+
+    def configure_get_context(self):
+        context = super(VendorCatalogView, self).configure_get_context()
+        app = self.get_rattail_app()
+        vendor_handler = app.get_vendor_handler()
+
+        Parsers = vendor_handler.get_all_catalog_parsers()
+        Supported = vendor_handler.get_supported_catalog_parsers()
+        context['catalog_parsers'] = Parsers
+        context['catalog_parsers_data'] = dict([(Parser.key, Parser in Supported)
+                                                for Parser in Parsers])
+
+        return context
+
+    def configure_gather_settings(self, data):
+        settings = super(VendorCatalogView, self).configure_gather_settings(data)
+        app = self.get_rattail_app()
+        vendor_handler = app.get_vendor_handler()
+
+        supported = []
+        for Parser in vendor_handler.get_all_catalog_parsers():
+            name = 'catalog_parser_{}'.format(Parser.key)
+            if data.get(name) == 'true':
+                supported.append(Parser.key)
+        settings.append({'name': 'rattail.vendors.supported_catalog_parsers',
+                         'value': ', '.join(supported)})
+
+        return settings
+
+    def configure_remove_settings(self):
+        super(VendorCatalogView, self).configure_remove_settings()
+        model = self.model
+        names = [
+            'rattail.vendors.supported_catalog_parsers',
+            'tailbone.batch.vendorcatalog.supported_parsers', # deprecated
+        ]
+
+        Session().query(model.Setting)\
+                 .filter(model.Setting.name.in_(names))\
+                 .delete(synchronize_session=False)
+
 
 # TODO: deprecate / remove this
 VendorCatalogsView = VendorCatalogView
