@@ -2,7 +2,7 @@
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2018 Lance Edgar
+#  Copyright © 2010-2021 Lance Edgar
 #
 #  This file is part of Rattail.
 #
@@ -147,6 +147,7 @@ class EmailBounceView(MasterView):
             kwargs['message'] = "(file not found)"
         return kwargs
 
+    # TODO: should require POST here
     def process(self):
         """
         View for marking a bounce as processed.
@@ -155,8 +156,9 @@ class EmailBounceView(MasterView):
         bounce.processed = datetime.datetime.utcnow()
         bounce.processed_by = self.request.user
         self.request.session.flash("Email bounce has been marked processed.")
-        return self.redirect(self.request.route_url('emailbounces'))
+        return self.redirect(self.get_action_url('view', bounce))
 
+    # TODO: should require POST here
     def unprocess(self):
         """
         View for marking a bounce as *unprocessed*.
@@ -165,7 +167,7 @@ class EmailBounceView(MasterView):
         bounce.processed = None
         bounce.processed_by = None
         self.request.session.flash("Email bounce has been marked UN-processed.")
-        return self.redirect(self.request.route_url('emailbounces'))
+        return self.redirect(self.get_action_url('view', bounce))
 
     def download(self):
         """
@@ -207,9 +209,6 @@ class EmailBounceView(MasterView):
 
         cls._defaults(config)
 
-# TODO: deprecate / remove this
-EmailBouncesView = EmailBounceView
-
 
 def includeme(config):
     EmailBounceView.defaults(config)
