@@ -2,7 +2,7 @@
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2020 Lance Edgar
+#  Copyright © 2010-2021 Lance Edgar
 #
 #  This file is part of Rattail.
 #
@@ -42,7 +42,7 @@ from tailbone.db import Session
 from tailbone.auth import logout_user
 from tailbone.progress import SessionProgress
 from tailbone.util import should_use_buefy
-from tailbone.config import legacy_mobile_enabled, protected_usernames
+from tailbone.config import protected_usernames
 
 
 class View(object):
@@ -68,6 +68,7 @@ class View(object):
         config = self.rattail_config
         if config:
             self.enum = config.get_enum()
+            self.model = config.get_model()
 
     @property
     def rattail_config(self):
@@ -100,14 +101,6 @@ class View(object):
         """
         return should_use_buefy(self.request)
 
-    @classmethod
-    def legacy_mobile_enabled(cls, rattail_config):
-        """
-        Returns the boolean setting indicating whether the old / "legacy"
-        (jQuery) mobile app/site should be exposed.
-        """
-        return legacy_mobile_enabled(rattail_config)
-
     def late_login_user(self):
         """
         Returns the :class:`rattail:rattail.db.model.User` instance
@@ -166,7 +159,7 @@ class View(object):
         return render_to_response('json', data,
                                   request=self.request)
 
-    def file_response(self, path, filename=None):
+    def file_response(self, path, filename=None, attachment=True):
         """
         Returns a generic FileResponse from the given path
         """
@@ -174,11 +167,12 @@ class View(object):
             return self.notfound()
         response = FileResponse(path, request=self.request)
         response.content_length = os.path.getsize(path)
-        if not filename:
-            filename = os.path.basename(path)
-            if six.PY2:
-                filename = filename.encode('ascii', 'replace')
-        response.content_disposition = str('attachment; filename="{}"'.format(filename))
+        if attachment:
+            if not filename:
+                filename = os.path.basename(path)
+                if six.PY2:
+                    filename = filename.encode('ascii', 'replace')
+            response.content_disposition = str('attachment; filename="{}"'.format(filename))
         return response
 
     def get_quickie_context(self):
