@@ -2,7 +2,7 @@
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2021 Lance Edgar
+#  Copyright © 2010-2022 Lance Edgar
 #
 #  This file is part of Rattail.
 #
@@ -112,7 +112,7 @@ class EmployeeView(MasterView):
             g.set_sorter('username', model.User.username)
             g.set_renderer('username', self.grid_render_username)
         else:
-            g.hide_column('username')
+            g.remove('username')
 
         # id
         if self.has_perm('edit'):
