@@ -27,21 +27,26 @@ Person Views
 from __future__ import unicode_literals, absolute_import
 
 import datetime
+import logging
 
 import six
 import sqlalchemy as sa
 from sqlalchemy import orm
 
 from rattail.db import model, api
+from rattail.db.util import maxlen
 from rattail.time import localtime
-from rattail.util import OrderedDict
+from rattail.util import OrderedDict, simple_error
 
 import colander
 from pyramid.httpexceptions import HTTPFound, HTTPNotFound
 from webhelpers2.html import HTML, tags
 
 from tailbone import forms, grids
-from tailbone.views import MasterView, AutocompleteView
+from tailbone.views import MasterView
+
+
+log = logging.getLogger(__name__)
 
 
 class PersonView(MasterView):
@@ -56,6 +61,7 @@ class PersonView(MasterView):
     bulk_deletable = True
     is_contact = True
     manage_notes_from_profile_view = False
+    supports_autocomplete = True
 
     labels = {
         'default_phone': "Phone Number",
@@ -68,6 +74,7 @@ class PersonView(MasterView):
         'last_name',
         'phone',
         'email',
+        'merge_requested',
     ]
 
     form_fields = [
@@ -85,16 +92,22 @@ class PersonView(MasterView):
     ]
 
     mergeable = True
-    merge_additive_fields = [
-        'usernames',
-        'member_uuids',
-    ]
-    merge_fields = merge_additive_fields + [
-        'uuid',
-        'first_name',
-        'last_name',
-        'display_name',
-    ]
+
+    def __init__(self, request):
+        super(PersonView, self).__init__(request)
+
+        # always get a reference to the People Handler
+        app = self.get_rattail_app()
+        self.handler = app.get_people_handler()
+
+    def make_grid_kwargs(self, **kwargs):
+        kwargs = super(PersonView, self).make_grid_kwargs(**kwargs)
+
+        # turn on checkboxes if user can create a merge reqeust
+        if self.mergeable and self.has_perm('request_merge'):
+            kwargs['checkboxes'] = True
+
+        return kwargs
 
     def configure_grid(self, g):
         super(PersonView, self).configure_grid(g)
@@ -126,6 +139,9 @@ class PersonView(MasterView):
         g.sorters['email'] = lambda q, d: q.order_by(getattr(model.PersonEmailAddress.address, d)())
         g.sorters['phone'] = lambda q, d: q.order_by(getattr(model.PersonPhoneNumber.number, d)())
 
+        g.set_label('merge_requested', "MR")
+        g.set_renderer('merge_requested', self.render_merge_requested)
+
         g.set_sort_defaults('display_name')
 
         g.set_label('display_name', "Full Name")
@@ -137,6 +153,23 @@ class PersonView(MasterView):
         g.set_link('first_name')
         g.set_link('last_name')
 
+    def render_merge_requested(self, person, field):
+        model = self.model
+        merge_request = self.Session.query(model.MergePeopleRequest)\
+                                    .filter(sa.or_(
+                                        model.MergePeopleRequest.removing_uuid == person.uuid,
+                                        model.MergePeopleRequest.keeping_uuid == person.uuid))\
+                                    .filter(model.MergePeopleRequest.merged == None)\
+                                    .first()
+        if merge_request:
+            use_buefy = self.get_use_buefy()
+            if use_buefy:
+                return HTML.tag('span',
+                                class_='has-text-danger has-text-weight-bold',
+                                title="A merge has been requested for this person.",
+                                c="MR")
+            return "MR"
+
     def get_instance(self):
         # TODO: I don't recall why this fallback check for a vendor contact
         # exists here, but leaving it intact for now.
@@ -190,9 +223,7 @@ class PersonView(MasterView):
                 names[key] = None
 
         # do explicit name update w/ common handler logic
-        app = self.get_rattail_app()
-        handler = app.get_people_handler()
-        handler.update_names(person, **names)
+        self.handler.update_names(person, **names)
 
         return person
 
@@ -365,33 +396,30 @@ class PersonView(MasterView):
             (model.VendorContact, 'person_uuid'),
         ]
 
+    def get_merge_fields(self):
+        fields = self.handler.get_merge_preview_fields()
+        return [field['name'] for field in fields]
+
+    def get_merge_additive_fields(self):
+        fields = self.handler.get_merge_preview_fields()
+        return [field['name'] for field in fields
+                if field.get('additive')]
+
+    def get_merge_coalesce_fields(self):
+        fields = self.handler.get_merge_preview_fields()
+        return [field['name'] for field in fields
+                if field.get('coalesce')]
+
     def get_merge_data(self, person):
-        return {
-            'uuid': person.uuid,
-            'first_name': person.first_name,
-            'last_name': person.last_name,
-            'display_name': person.display_name,
-            'usernames': [u.username for u in person.users],
-            'member_uuids': [m.uuid for m in person.members],
-        }
+        return self.handler.get_merge_preview_data(person)
+
+    def validate_merge(self, removing, keeping):
+        reason = self.handler.why_not_merge(removing, keeping)
+        if reason:
+            raise Exception(reason)
 
     def merge_objects(self, removing, keeping):
-        """
-        Execute a merge operation on the two given person records.
-        """
-        # move Member records to final Person
-        for member in list(removing.members):
-            removing.members.remove(member)
-            keeping.members.append(member)
-
-        # move User records to final Person
-        for user in list(removing.users):
-            removing.users.remove(user)
-            keeping.users.append(user)
-
-        # delete unwanted Person
-        self.Session.delete(removing)
-        self.Session.flush()
+        self.handler.perform_merge(removing, keeping, user=self.request.user)
 
     def view_profile(self):
         """
@@ -407,30 +435,100 @@ class PersonView(MasterView):
             'instance_title': self.get_instance_title(person),
             'today': localtime(self.rattail_config).date(),
             'person_data': self.get_context_person(person),
+            'phone_type_options': self.get_phone_type_options(),
+            'email_type_options': self.get_email_type_options(),
+            'max_lengths': self.get_max_lengths(),
             'customers_data': self.get_context_customers(person),
             'members_data': self.get_context_members(person),
             'employee': employee,
+            'employee_data': self.get_context_employee(employee) if employee else {},
             'employee_view_url': self.request.route_url('employees.view', uuid=employee.uuid) if employee else None,
             'employee_history': employee.get_current_history() if employee else None,
             'employee_history_data': self.get_context_employee_history(employee),
+            'dynamic_content_title': self.get_context_content_title(person),
         }
 
         use_buefy = self.get_use_buefy()
         template = 'view_profile_buefy' if use_buefy else 'view_profile'
         return self.render_to_response(template, context)
 
-    def get_context_person(self, person):
+    def template_kwargs_view_profile(self, **kwargs):
+        """
+        Stub method so subclass can call `super()` for it.
+        """
+        return kwargs
+
+    def template_kwargs_view_profile_buefy(self, **kwargs):
+        """
+        Note that any subclass should not need to define this method.
+        It by default invokes :meth:`template_kwargs_view_profile()`
+        and returns that result.
+        """
+        return self.template_kwargs_view_profile(**kwargs)
+
+    def get_max_lengths(self):
+        model = self.model
         return {
+            'address_street': maxlen(model.PersonMailingAddress.street),
+            'address_street2': maxlen(model.PersonMailingAddress.street2),
+            'address_city': maxlen(model.PersonMailingAddress.city),
+            'address_state': maxlen(model.PersonMailingAddress.state),
+            'address_zipcode': maxlen(model.PersonMailingAddress.zipcode),
+        }
+
+    def get_phone_type_options(self):
+        """
+        Returns a list of "phone type" options, for use in dropdown.
+        """
+        # TODO: should probably define this list somewhere else
+        phone_types = [
+            "Home",
+            "Mobile",
+            "Work",
+            "Other",
+            "Fax",
+        ]
+        return [{'value': typ, 'label': typ}
+                for typ in phone_types]
+
+    def get_email_type_options(self):
+        """
+        Returns a list of "email type" options, for use in dropdown.
+        """
+        # TODO: should probably define this list somewhere else
+        email_types = [
+            "Home",
+            "Work",
+            "Other",
+        ]
+        return [{'value': typ, 'label': typ}
+                for typ in email_types]
+
+    def get_context_person(self, person):
+
+        context = {
             'uuid': person.uuid,
             'first_name': person.first_name,
+            'middle_name': person.middle_name,
             'last_name': person.last_name,
             'display_name': person.display_name,
             'view_url': self.get_action_url('view', person),
             'view_profile_url': self.get_action_url('view_profile', person),
+            'phones': self.get_context_phones(person),
+            'emails': self.get_context_emails(person),
+            'dynamic_content_title': self.get_context_content_title(person),
         }
 
+        if person.address:
+            context['address'] = self.get_context_address(person.address)
+
+        return context
+
+    def get_context_content_title(self, person):
+        return six.text_type(person)
+
     def get_context_address(self, address):
-        return {
+        context = {
             'uuid': address.uuid,
             'street': address.street,
             'street2': address.street2,
@@ -440,6 +538,13 @@ class PersonView(MasterView):
             'display': six.text_type(address),
         }
 
+        model = self.model
+        if isinstance(address, model.PersonMailingAddress):
+            person = address.person
+            context['invalid'] = self.handler.address_is_invalid(person, address)
+
+        return context
+
     def get_context_customers(self, person):
         data = []
         for cp in person._customers:
@@ -501,6 +606,7 @@ class PersonView(MasterView):
         app = self.get_rattail_app()
         handler = app.get_employment_handler()
         context = handler.get_context_employee(employee)
+        context['view_url'] = self.request.route_url('employees.view', uuid=employee.uuid)
         return context
 
     def get_context_employee_history(self, employee):
@@ -524,6 +630,271 @@ class PersonView(MasterView):
         customer = handler.ensure_customer(person)
         return customer
 
+    def profile_edit_name(self):
+        """
+        View which allows a person's name to be updated.
+        """
+        person = self.get_instance()
+        data = dict(self.request.json_body)
+
+        self.handler.update_names(person,
+                                  first=data['first_name'],
+                                  middle=data['middle_name'],
+                                  last=data['last_name'])
+
+        self.Session.flush()
+        return {
+            'success': True,
+            'person': self.get_context_person(person),
+            'dynamic_content_title': self.get_context_content_title(person),
+        }
+
+    def get_context_phones(self, person):
+        data = []
+        for phone in person.phones:
+            data.append({
+                'uuid': phone.uuid,
+                'type': phone.type,
+                'number': phone.number,
+                'preferred': phone.preferred,
+                'preference': phone.preference,
+            })
+        return data
+
+    def profile_add_phone(self):
+        """
+        View which adds a new phone number for the person.
+        """
+        person = self.get_instance()
+        data = dict(self.request.json_body)
+
+        try:
+            phone = self.handler.add_phone(person, data['phone_number'],
+                                           type=data['phone_type'],
+                                           preferred=data['phone_preferred'])
+        except Exception as error:
+            log.warning("failed to add phone", exc_info=True)
+            return {'error': simple_error(error)}
+
+        self.Session.flush()
+        return {
+            'success': True,
+            'person': self.get_context_person(person),
+        }
+
+    def profile_update_phone(self):
+        """
+        View which updates a phone number for the person.
+        """
+        person = self.get_instance()
+        data = dict(self.request.json_body)
+
+        phone = self.Session.query(model.PersonPhoneNumber).get(data['phone_uuid'])
+        if not phone:
+            return {'error': "Phone not found."}
+
+        kwargs = {
+            'number': data['phone_number'],
+            'type': data['phone_type'],
+        }
+        if 'phone_preferred' in data:
+            kwargs['preferred'] = data['phone_preferred']
+
+        try:
+            phone = self.handler.update_phone(person, phone, **kwargs)
+        except Exception as error:
+            log.warning("failed to update phone", exc_info=True)
+            return {'error': simple_error(error)}
+
+        self.Session.flush()
+        return {
+            'success': True,
+            'person': self.get_context_person(person),
+        }
+
+    def profile_delete_phone(self):
+        """
+        View which allows a person's phone number to be deleted.
+        """
+        person = self.get_instance()
+        data = dict(self.request.json_body)
+
+        # validate phone
+        phone = self.Session.query(model.PersonPhoneNumber).get(data['phone_uuid'])
+        if not phone:
+            return {'error': "Phone not found."}
+        if phone not in person.phones:
+            return {'error': "Phone does not belong to this person."}
+
+        # remove phone
+        person.remove_phone(phone)
+
+        self.Session.flush()
+        return {
+            'success': True,
+            'person': self.get_context_person(person),
+        }
+
+    def profile_set_preferred_phone(self):
+        """
+        View which allows a person's "preferred" phone to be set.
+        """
+        person = self.get_instance()
+        data = dict(self.request.json_body)
+
+        # validate phone
+        phone = self.Session.query(model.PersonPhoneNumber).get(data['phone_uuid'])
+        if not phone:
+            return {'error': "Phone not found."}
+        if phone not in person.phones:
+            return {'error': "Phone does not belong to this person."}
+
+        # update phone preference
+        person.set_primary_phone(phone)
+
+        self.Session.flush()
+        return {
+            'success': True,
+            'person': self.get_context_person(person),
+        }
+
+    def get_context_emails(self, person):
+        data = []
+        for email in person.emails:
+            data.append({
+                'uuid': email.uuid,
+                'type': email.type,
+                'address': email.address,
+                'invalid': email.invalid,
+                'preferred': email.preferred,
+                'preference': email.preference,
+            })
+        return data
+
+    def profile_add_email(self):
+        """
+        View which adds a new email address for the person.
+        """
+        person = self.get_instance()
+        data = dict(self.request.json_body)
+
+        kwargs = {
+            'type': data['email_type'],
+            'invalid': False,
+        }
+        if 'email_preferred' in data:
+            kwargs['preferred'] = data['email_preferred']
+
+        try:
+            email = self.handler.add_email(person, data['email_address'], **kwargs)
+        except Exception as error:
+            log.warning("failed to add email", exc_info=True)
+            return {'error': simple_error(error)}
+
+        self.Session.flush()
+        return {
+            'success': True,
+            'person': self.get_context_person(person),
+        }
+
+    def profile_update_email(self):
+        """
+        View which updates an email address for the person.
+        """
+        person = self.get_instance()
+        data = dict(self.request.json_body)
+
+        email = self.Session.query(model.PersonEmailAddress).get(data['email_uuid'])
+        if not email:
+            return {'error': "Email not found."}
+
+        try:
+            email = self.handler.update_email(person, email,
+                                              address=data['email_address'],
+                                              type=data['email_type'],
+                                              invalid=data['email_invalid'])
+        except Exception as error:
+            log.warning("failed to add email", exc_info=True)
+            return {'error': simple_error(error)}
+
+        self.Session.flush()
+        return {
+            'success': True,
+            'person': self.get_context_person(person),
+        }
+
+    def profile_delete_email(self):
+        """
+        View which allows a person's email address to be deleted.
+        """
+        person = self.get_instance()
+        data = dict(self.request.json_body)
+
+        # validate email
+        email = self.Session.query(model.PersonEmailAddress).get(data['email_uuid'])
+        if not email:
+            return {'error': "Email not found."}
+        if email not in person.emails:
+            return {'error': "Email does not belong to this person."}
+
+        # remove email
+        person.remove_email(email)
+
+        self.Session.flush()
+
+        return {
+            'success': True,
+            'person': self.get_context_person(person),
+        }
+
+    def profile_set_preferred_email(self):
+        """
+        View which allows a person's "preferred" email to be set.
+        """
+        person = self.get_instance()
+        data = dict(self.request.json_body)
+
+        # validate email
+        email = self.Session.query(model.PersonEmailAddress).get(data['email_uuid'])
+        if not email:
+            return {'error': "Email not found."}
+        if email not in person.emails:
+            return {'error': "Email does not belong to this person."}
+
+        # update email preference
+        person.set_primary_email(email)
+
+        self.Session.flush()
+        return {
+            'success': True,
+            'person': self.get_context_person(person),
+        }
+
+    def profile_edit_address(self):
+        """
+        View which allows a person's mailing address to be updated.
+        """
+        person = self.get_instance()
+        data = dict(self.request.json_body)
+
+        # update person address
+        address = person.address
+        if not address:
+            address = person.add_address()
+        address.street = data['street']
+        address.street2 = data['street2']
+        address.city = data['city']
+        address.state = data['state']
+        address.zipcode = data['zipcode']
+
+        self.handler.mark_address_invalid(person, address, data['invalid'])
+
+        self.Session.flush()
+        return {
+            'success': True,
+            'person': self.get_context_person(person),
+        }
+
     def profile_start_employee(self):
         """
         View which will cause the person to start being an employee.
@@ -540,6 +911,7 @@ class PersonView(MasterView):
         start_date = datetime.datetime.strptime(data['start_date'], '%Y-%m-%d').date()
         employee = handler.begin_employment(person, start_date,
                                             employee_id=data['id'])
+        self.Session.flush()
         return self.profile_start_employee_result(employee, start_date)
 
     def profile_start_employee_result(self, employee, start_date):
@@ -568,6 +940,7 @@ class PersonView(MasterView):
         employee = handler.get_employee(person)
         handler.end_employment(employee, end_date,
                                revoke_access=data.get('revoke_access'))
+        self.Session.flush()
         return self.profile_end_employee_result(employee, end_date)
 
     def profile_end_employee_result(self, employee, end_date):
@@ -604,14 +977,33 @@ class PersonView(MasterView):
 
         self.Session.flush()
         current_history = employee.get_current_history()
-
         return {
             'success': True,
+            'employee': self.get_context_employee(employee),
             'start_date': six.text_type(current_history.start_date),
             'end_date': six.text_type(current_history.end_date or ''),
             'employee_history_data': self.get_context_employee_history(employee),
         }
 
+    def profile_update_employee_id(self):
+        """
+        View to update an employee's ID value.
+        """
+        app = self.get_rattail_app()
+        employment = app.get_employment_handler()
+
+        person = self.get_instance()
+        employee = employment.get_employee(person)
+
+        data = self.request.json_body
+        employee.id = data['employee_id']
+        self.Session.flush()
+
+        return {
+            'success': True,
+            'employee': self.get_context_employee(employee),
+        }
+
     def make_note_form(self, mode, person):
         schema = NoteSchema().bind(session=self.Session(),
                                    person_uuid=person.uuid)
@@ -704,6 +1096,15 @@ class PersonView(MasterView):
         self.request.session.flash("User has been created: {}".format(user.username))
         return self.redirect(self.request.route_url('users.view', uuid=user.uuid))
 
+    def request_merge(self):
+        """
+        Create a new merge request for the given 2 people.
+        """
+        self.handler.request_merge(self.request.user,
+                                   self.request.POST['removing_uuid'],
+                                   self.request.POST['keeping_uuid'])
+        return self.redirect(self.get_index_url())
+
     @classmethod
     def defaults(cls, config):
         cls._people_defaults(config)
@@ -717,6 +1118,7 @@ class PersonView(MasterView):
         instance_url_prefix = cls.get_instance_url_prefix()
         model_key = cls.get_model_key()
         model_title = cls.get_model_title()
+        model_title_plural = cls.get_model_title_plural()
 
         # "profile" perms
         # TODO: should let view class (or config) determine which of these are available
@@ -734,6 +1136,101 @@ class PersonView(MasterView):
         config.add_view(cls, attr='view_profile', route_name='{}.view_profile'.format(route_prefix),
                         permission='{}.view_profile'.format(permission_prefix))
 
+        # profile - edit personal details
+        config.add_tailbone_permission('people_profile',
+                                       'people_profile.edit_person',
+                                       "Edit the Personal details")
+
+        # profile - edit name
+        config.add_route('{}.profile_edit_name'.format(route_prefix),
+                         '{}/profile/edit-name'.format(instance_url_prefix),
+                         request_method='POST')
+        config.add_view(cls, attr='profile_edit_name',
+                        route_name='{}.profile_edit_name'.format(route_prefix),
+                        renderer='json',
+                        permission='people_profile.edit_person')
+
+        # profile - add phone
+        config.add_route('{}.profile_add_phone'.format(route_prefix),
+                         '{}/profile/add-phone'.format(instance_url_prefix),
+                         request_method='POST')
+        config.add_view(cls, attr='profile_add_phone',
+                        route_name='{}.profile_add_phone'.format(route_prefix),
+                        renderer='json',
+                        permission='people_profile.edit_person')
+
+        # profile - update phone
+        config.add_route('{}.profile_update_phone'.format(route_prefix),
+                         '{}/profile/update-phone'.format(instance_url_prefix),
+                         request_method='POST')
+        config.add_view(cls, attr='profile_update_phone',
+                        route_name='{}.profile_update_phone'.format(route_prefix),
+                        renderer='json',
+                        permission='people_profile.edit_person')
+
+        # profile - delete phone
+        config.add_route('{}.profile_delete_phone'.format(route_prefix),
+                         '{}/profile/delete-phone'.format(instance_url_prefix),
+                         request_method='POST')
+        config.add_view(cls, attr='profile_delete_phone',
+                        route_name='{}.profile_delete_phone'.format(route_prefix),
+                        renderer='json',
+                        permission='people_profile.edit_person')
+
+        # profile - set preferred phone
+        config.add_route('{}.profile_set_preferred_phone'.format(route_prefix),
+                         '{}/profile/set-preferred-phone'.format(instance_url_prefix),
+                         request_method='POST')
+        config.add_view(cls, attr='profile_set_preferred_phone',
+                        route_name='{}.profile_set_preferred_phone'.format(route_prefix),
+                        renderer='json',
+                        permission='people_profile.edit_person')
+
+        # profile - add email
+        config.add_route('{}.profile_add_email'.format(route_prefix),
+                         '{}/profile/add-email'.format(instance_url_prefix),
+                         request_method='POST')
+        config.add_view(cls, attr='profile_add_email',
+                        route_name='{}.profile_add_email'.format(route_prefix),
+                        renderer='json',
+                        permission='people_profile.edit_person')
+
+        # profile - update email
+        config.add_route('{}.profile_update_email'.format(route_prefix),
+                         '{}/profile/update-email'.format(instance_url_prefix),
+                         request_method='POST')
+        config.add_view(cls, attr='profile_update_email',
+                        route_name='{}.profile_update_email'.format(route_prefix),
+                        renderer='json',
+                        permission='people_profile.edit_person')
+
+        # profile - delete email
+        config.add_route('{}.profile_delete_email'.format(route_prefix),
+                         '{}/profile/delete-email'.format(instance_url_prefix),
+                         request_method='POST')
+        config.add_view(cls, attr='profile_delete_email',
+                        route_name='{}.profile_delete_email'.format(route_prefix),
+                        renderer='json',
+                        permission='people_profile.edit_person')
+
+        # profile - set preferred email
+        config.add_route('{}.profile_set_preferred_email'.format(route_prefix),
+                         '{}/profile/set-preferred-email'.format(instance_url_prefix),
+                         request_method='POST')
+        config.add_view(cls, attr='profile_set_preferred_email',
+                        route_name='{}.profile_set_preferred_email'.format(route_prefix),
+                        renderer='json',
+                        permission='people_profile.edit_person')
+
+        # profile - edit address
+        config.add_route('{}.profile_edit_address'.format(route_prefix),
+                         '{}/profile/edit-address'.format(instance_url_prefix),
+                         request_method='POST')
+        config.add_view(cls, attr='profile_edit_address',
+                        route_name='{}.profile_edit_address'.format(route_prefix),
+                        renderer='json',
+                        permission='people_profile.edit_person')
+
         # profile - start employee
         config.add_route('{}.profile_start_employee'.format(route_prefix), '{}/profile/start-employee'.format(instance_url_prefix),
                          request_method='POST')
@@ -752,6 +1249,15 @@ class PersonView(MasterView):
         config.add_view(cls, attr='profile_edit_employee_history', route_name='{}.profile_edit_employee_history'.format(route_prefix),
                         permission='people_profile.edit_employee_history', renderer='json')
 
+        # profile - update employee ID
+        config.add_route('{}.profile_update_employee_id'.format(route_prefix),
+                         '{}/profile/update-employee-id'.format(instance_url_prefix),
+                         request_method='POST')
+        config.add_view(cls, attr='profile_update_employee_id',
+                        route_name='{}.profile_update_employee_id'.format(route_prefix),
+                        renderer='json',
+                        permission='employees.edit')
+
         # manage notes from profile view
         if cls.manage_notes_from_profile_view:
 
@@ -785,24 +1291,14 @@ class PersonView(MasterView):
         config.add_view(cls, attr='make_user', route_name='{}.make_user'.format(route_prefix),
                         permission='users.create')
 
-# TODO: deprecate / remove this
-PeopleView = PersonView
-
-
-class PeopleAutocomplete(AutocompleteView):
-
-    mapped_class = model.Person
-    fieldname = 'display_name'
-
-
-class PeopleEmployeesAutocomplete(PeopleAutocomplete):
-    """
-    Autocomplete view for the Person model, but restricted to return only
-    results for people who are employees.
-    """
-
-    def filter_query(self, q):
-        return q.join(model.Employee)
+        # merge requests
+        if cls.mergeable:
+            config.add_tailbone_permission(permission_prefix, '{}.request_merge'.format(permission_prefix),
+                                           "Request merge for 2 {}".format(model_title_plural))
+            config.add_route('{}.request_merge'.format(route_prefix), '{}/request-merge'.format(url_prefix),
+                             request_method='POST')
+            config.add_view(cls, attr='request_merge', route_name='{}.request_merge'.format(route_prefix),
+                            permission='{}.request_merge'.format(permission_prefix))
 
 
 class PersonNoteView(MasterView):
@@ -896,15 +1392,85 @@ class NoteSchema(colander.Schema):
     note_text = colander.SchemaNode(colander.String(), missing='')
 
 
+class MergePeopleRequestView(MasterView):
+    """
+    Master view for the MergePeopleRequest class.
+    """
+    model_class = model.MergePeopleRequest
+    route_prefix = 'people_merge_requests'
+    url_prefix = '/people/merge-requests'
+    creatable = False
+    editable = False
+
+    labels = {
+        'removing_uuid': "Removing",
+        'keeping_uuid': "Keeping",
+    }
+
+    grid_columns = [
+        'removing_uuid',
+        'keeping_uuid',
+        'requested',
+        'requested_by',
+        'merged',
+        'merged_by',
+    ]
+
+    form_fields = [
+        'removing_uuid',
+        'keeping_uuid',
+        'requested',
+        'requested_by',
+        'merged',
+        'merged_by',
+    ]
+
+    def configure_grid(self, g):
+        super(MergePeopleRequestView, self).configure_grid(g)
+
+        g.set_renderer('removing_uuid', self.render_referenced_person_name)
+        g.set_renderer('keeping_uuid', self.render_referenced_person_name)
+
+        g.filters['merged'].default_active = True
+        g.filters['merged'].default_verb = 'is_null'
+
+        g.set_sort_defaults('requested', 'desc')
+
+        g.set_link('removing_uuid')
+        g.set_link('keeping_uuid')
+
+    def render_referenced_person_name(self, merge_request, field):
+        uuid = getattr(merge_request, field)
+        person = self.Session.query(self.model.Person).get(uuid)
+        if person:
+            return six.text_type(person)
+        return "(person not found)"
+
+    def get_instance_title(self, merge_request):
+        model = self.model
+        removing = self.Session.query(model.Person).get(merge_request.removing_uuid)
+        keeping = self.Session.query(model.Person).get(merge_request.keeping_uuid)
+        return "{} -> {}".format(
+            removing or "(not found)",
+            keeping or "(not found)")
+
+    def configure_form(self, f):
+        super(MergePeopleRequestView, self).configure_form(f)
+
+        f.set_renderer('removing_uuid', self.render_referenced_person)
+        f.set_renderer('keeping_uuid', self.render_referenced_person)
+
+    def render_referenced_person(self, merge_request, field):
+        uuid = getattr(merge_request, field)
+        person = self.Session.query(self.model.Person).get(uuid)
+        if person:
+            text = six.text_type(person)
+            url = self.request.route_url('people.view', uuid=person.uuid)
+            return tags.link_to(text, url)
+        return "(person not found)"
+
+
 def includeme(config):
-
-    # autocomplete
-    config.add_route('people.autocomplete', '/people/autocomplete')
-    config.add_view(PeopleAutocomplete, route_name='people.autocomplete',
-                    renderer='json', permission='people.list')
-    config.add_route('people.autocomplete.employees', '/people/autocomplete/employees')
-    config.add_view(PeopleEmployeesAutocomplete, route_name='people.autocomplete.employees',
-                    renderer='json', permission='people.list')
-
     PersonView.defaults(config)
     PersonNoteView.defaults(config)
+    MergePeopleRequestView.defaults(config)
