@@ -2,7 +2,7 @@
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2021 Lance Edgar
+#  Copyright © 2010-2022 Lance Edgar
 #
 #  This file is part of Rattail.
 #
@@ -41,7 +41,6 @@ from rattail.gpc import GPC
 from rattail.threads import Thread
 from rattail.exceptions import LabelPrintingError
 from rattail.util import load_object, pretty_quantity, OrderedDict, simple_error
-from rattail.batch import get_batch_handler
 from rattail.time import localtime, make_utc
 
 import colander
@@ -50,7 +49,6 @@ from pyramid import httpexceptions
 from webhelpers2.html import tags, HTML
 
 from tailbone import forms, grids
-from tailbone.db import Session
 from tailbone.views import MasterView
 from tailbone.util import raw_datetime
 
@@ -179,10 +177,13 @@ class ProductView(MasterView):
 
     def __init__(self, request):
         super(ProductView, self).__init__(request)
-        self.print_labels = request.rattail_config.getbool('tailbone', 'products.print_labels', default=False)
+        self.expose_label_printing = self.rattail_config.getbool(
+            'tailbone', 'products.print_labels', default=False)
 
         app = self.get_rattail_app()
-        self.handler = app.get_products_handler()
+        self.product_handler = app.get_products_handler()
+        # TODO: deprecate / remove this
+        self.handler = self.product_handler
 
     def query(self, session):
         user = self.request.user
@@ -210,6 +211,18 @@ class ProductView(MasterView):
 
         return query
 
+    def get_departments(self):
+        """
+        Returns the list of departments to be exposed in a drop-down.
+        """
+        model = self.model
+        return self.Session.query(model.Department)\
+                           .filter(sa.or_(
+                               model.Department.product == True,
+                               model.Department.product == None))\
+                           .order_by(model.Department.name)\
+                           .all()
+
     def configure_grid(self, g):
         super(ProductView, self).configure_grid(g)
         app = self.get_rattail_app()
@@ -246,12 +259,9 @@ class ProductView(MasterView):
         # department
         g.set_joiner('department', lambda q: q.outerjoin(model.Department))
         g.set_sorter('department', model.Department.name)
-        department_choices = app.cache_model(self.Session(), model.Department,
-                                             order_by=model.Department.name,
-                                             normalizer=lambda d: d.name)
+        departments = self.get_departments()
         department_choices = OrderedDict([('', "(any)")]
-                                         + sorted(six.iteritems(department_choices),
-                                                  key=lambda itm: itm[1]))
+                                         + [(d.uuid, d.name) for d in departments])
         if not use_buefy:
             department_choices = [tags.Option(name, uuid)
                                   for uuid, name in six.iteritems(department_choices)]
@@ -359,8 +369,13 @@ class ProductView(MasterView):
 
         g.set_sort_defaults('upc')
 
-        if self.print_labels and self.request.has_perm('products.print_labels'):
-            g.more_actions.append(grids.GridAction('print_label', icon='print'))
+        if self.expose_label_printing and self.has_perm('print_labels'):
+            if use_buefy:
+                g.more_actions.append(self.make_action(
+                    'print_label', icon='print', url='#',
+                    click_handler='quickLabelPrint(props.row)'))
+            else:
+                g.more_actions.append(grids.GridAction('print_label', icon='print'))
 
         g.set_type('upc', 'gpc')
 
@@ -523,7 +538,7 @@ class ProductView(MasterView):
         if not product.not_for_sale:
             price = product[field]
             if price:
-                return self.handler.render_price(price)
+                return self.product_handler.render_price(price)
         
     def render_current_price_for_grid(self, product, field):
         text = self.render_price(product, field) or ""
@@ -652,13 +667,20 @@ class ProductView(MasterView):
         return pretty_quantity(inventory.on_order)
 
     def template_kwargs_index(self, **kwargs):
-        if self.print_labels:
-            kwargs['label_profiles'] = Session.query(model.LabelProfile)\
-                                              .filter(model.LabelProfile.visible == True)\
-                                              .order_by(model.LabelProfile.ordinal)\
-                                              .all()
-        return kwargs
+        kwargs = super(ProductView, self).template_kwargs_index(**kwargs)
+        model = self.model
 
+        if self.expose_label_printing:
+
+            kwargs['label_profiles'] = self.Session.query(model.LabelProfile)\
+                                                   .filter(model.LabelProfile.visible == True)\
+                                                   .order_by(model.LabelProfile.ordinal)\
+                                                   .all()
+
+            kwargs['quick_label_speedbump_threshold'] = self.rattail_config.getint(
+                'tailbone', 'products.quick_labels.speedbump_threshold')
+
+        return kwargs
 
     def grid_extra_class(self, product, i):
         classes = []
@@ -795,10 +817,10 @@ class ProductView(MasterView):
 
     def get_instance(self):
         key = self.request.matchdict['uuid']
-        product = Session.query(model.Product).get(key)
+        product = self.Session.query(model.Product).get(key)
         if product:
             return product
-        price = Session.query(model.ProductPrice).get(key)
+        price = self.Session.query(model.ProductPrice).get(key)
         if price:
             return price.product
         raise httpexceptions.HTTPNotFound()
@@ -811,8 +833,7 @@ class ProductView(MasterView):
         if self.creating or self.editing:
             if 'department' in f.fields:
                 f.replace('department', 'department_uuid')
-                departments = self.Session.query(model.Department)\
-                                          .order_by(model.Department.number)
+                departments = self.get_departments()
                 dept_values = [(d.uuid, "{} {}".format(d.number, d.name))
                                for d in departments]
                 require_department = False
@@ -1152,7 +1173,7 @@ class ProductView(MasterView):
         product = kwargs['instance']
         use_buefy = self.get_use_buefy()
 
-        kwargs['image_url'] = self.handler.get_image_url(product)
+        kwargs['image_url'] = self.product_handler.get_image_url(product)
         kwargs['product_key_field'] = self.rattail_config.product_key()
 
         # add price history, if user has access
@@ -1691,6 +1712,37 @@ class ProductView(MasterView):
         self.request.response.body = product.image.bytes
         return self.request.response
 
+    def print_labels(self):
+        app = self.get_rattail_app()
+        label_handler = app.get_label_handler()
+        model = self.model
+
+        profile = self.request.params.get('profile')
+        profile = self.Session.query(model.LabelProfile).get(profile) if profile else None
+        if not profile:
+            return {'error': "Label profile not found"}
+
+        product = self.request.params.get('product')
+        product = self.Session.query(model.Product).get(product) if product else None
+        if not product:
+            return {'error': "Product not found"}
+
+        quantity = self.request.params.get('quantity')
+        if not quantity.isdigit():
+            return {'error': "Quantity must be numeric"}
+        quantity = int(quantity)
+
+        printer = label_handler.get_printer(profile)
+        if not printer:
+            return {'error': "Couldn't get printer from label profile"}
+
+        try:
+            printer.print_labels([({'product': product}, quantity)])
+        except Exception as error:
+            log.warning("error occurred while printing labels", exc_info=True)
+            return {'error': six.text_type(error)}
+        return {'ok': True}
+
     def search(self):
         """
         Locate a product(s) by UPC.
@@ -1702,11 +1754,11 @@ class ProductView(MasterView):
         upc = self.request.GET.get('upc', '').strip()
         upc = re.sub(r'\D', '', upc)
         if upc:
-            product = api.get_product_by_upc(Session(), upc)
+            product = api.get_product_by_upc(self.Session(), upc)
             if not product:
                 # Try again, assuming caller did not include check digit.
                 upc = GPC(upc, calc_check_digit='upc')
-                product = api.get_product_by_upc(Session(), upc)
+                product = api.get_product_by_upc(self.Session(), upc)
             if product and (not product.deleted or self.request.has_perm('products.view_deleted')):
                 data = {
                     'uuid': product.uuid,
@@ -1717,7 +1769,7 @@ class ProductView(MasterView):
                 }
                 uuid = self.request.GET.get('with_vendor_cost')
                 if uuid:
-                    vendor = Session.query(model.Vendor).get(uuid)
+                    vendor = self.Session.query(model.Vendor).get(uuid)
                     if not vendor:
                         return {'error': "Vendor not found"}
                     cost = product.cost_for_vendor(vendor)
@@ -1913,21 +1965,28 @@ class ProductView(MasterView):
     def configure_get_simple_settings(self):
         return [
 
-            # key field
+            # display
             {'section': 'rattail',
              'option': 'product.key'},
             {'section': 'rattail',
              'option': 'product.key_title'},
+            {'section': 'tailbone',
+             'option': 'products.show_pod_image',
+             'type': bool},
 
             # handling
             {'section': 'rattail',
              'option': 'products.convert_type2_for_gpc_lookup',
              'type': bool},
 
-            # display
+            # labels
             {'section': 'tailbone',
-             'option': 'products.show_pod_image',
+             'option': 'products.print_labels',
              'type': bool},
+            {'section': 'tailbone',
+             'option': 'products.quick_labels.speedbump_threshold',
+             'type': int},
+
         ]
 
     @classmethod
@@ -1944,10 +2003,18 @@ class ProductView(MasterView):
         template_prefix = cls.get_template_prefix()
         permission_prefix = cls.get_permission_prefix()
         model_title = cls.get_model_title()
+        model_title_plural = cls.get_model_title_plural()
 
         # print labels
-        config.add_tailbone_permission('products', 'products.print_labels',
-                                       "Print labels for products")
+        config.add_tailbone_permission(permission_prefix,
+                                       '{}.print_labels'.format(permission_prefix),
+                                       "Print labels for {}".format(model_title_plural))
+        config.add_route('{}.print_labels'.format(route_prefix),
+                         '{}/labels'.format(url_prefix))
+        config.add_view(cls, attr='print_labels',
+                        route_name='{}.print_labels'.format(route_prefix),
+                        permission='{}.print_labels'.format(permission_prefix),
+                        renderer='json')
 
         # view deleted products
         config.add_tailbone_permission('products', 'products.view_deleted',
@@ -2230,39 +2297,6 @@ class PendingProductView(MasterView):
                         permission='{}.resolve_product'.format(permission_prefix))
 
 
-def print_labels(request):
-    profile = request.params.get('profile')
-    profile = Session.query(model.LabelProfile).get(profile) if profile else None
-    if not profile:
-        return {'error': "Label profile not found"}
-
-    product = request.params.get('product')
-    product = Session.query(model.Product).get(product) if product else None
-    if not product:
-        return {'error': "Product not found"}
-
-    quantity = request.params.get('quantity')
-    if not quantity.isdigit():
-        return {'error': "Quantity must be numeric"}
-    quantity = int(quantity)
-
-    printer = profile.get_printer(request.rattail_config)
-    if not printer:
-        return {'error': "Couldn't get printer from label profile"}
-
-    try:
-        printer.print_labels([(product, quantity, {})])
-    except Exception as error:
-        log.warning("error occurred while printing labels", exc_info=True)
-        return {'error': six.text_type(error)}
-    return {}
-
-
 def includeme(config):
-
-    config.add_route('products.print_labels', '/products/labels')
-    config.add_view(print_labels, route_name='products.print_labels',
-                    renderer='json', permission='products.print_labels')
-
     ProductView.defaults(config)
     PendingProductView.defaults(config)
