@@ -2,7 +2,7 @@
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2021 Lance Edgar
+#  Copyright © 2010-2022 Lance Edgar
 #
 #  This file is part of Rattail.
 #
@@ -294,9 +294,19 @@ class ReceivingBatchView(PurchasingBatchView):
                           use_buefy=use_buefy)
 
         # configure vendor field
-        use_autocomplete = self.rattail_config.getbool(
-            'rattail', 'vendor.use_autocomplete', default=True)
-        if use_autocomplete:
+        app = self.get_rattail_app()
+        vendor_handler = app.get_vendor_handler()
+        use_dropdown = vendor_handler.choice_uses_dropdown()
+        if use_dropdown:
+            vendors = self.Session.query(model.Vendor)\
+                                  .order_by(model.Vendor.id)
+            vendor_values = [(vendor.uuid, "({}) {}".format(vendor.id, vendor.name))
+                             for vendor in vendors]
+            if use_buefy:
+                form.set_widget('vendor', dfwidget.SelectWidget(values=vendor_values))
+            else:
+                form.set_widget('vendor', forms.widgets.JQuerySelectWidget(values=vendor_values))
+        else:
             vendor_display = ""
             if self.request.method == 'POST':
                 if self.request.POST.get('vendor'):
@@ -306,15 +316,6 @@ class ReceivingBatchView(PurchasingBatchView):
             vendors_url = self.request.route_url('vendors.autocomplete')
             form.set_widget('vendor', forms.widgets.JQueryAutocompleteWidget(
                 field_display=vendor_display, service_url=vendors_url))
-        else:
-            vendors = self.Session.query(model.Vendor)\
-                                  .order_by(model.Vendor.id)
-            vendor_values = [(vendor.uuid, "({}) {}".format(vendor.id, vendor.name))
-                             for vendor in vendors]
-            if use_buefy:
-                form.set_widget('vendor', dfwidget.SelectWidget(values=vendor_values))
-            else:
-                form.set_widget('vendor', forms.widgets.JQuerySelectWidget(values=vendor_values))
 
         # configure workflow field
         values = [(workflow['workflow_key'], workflow['display'])
