@@ -50,6 +50,24 @@ class ReportCodeView(MasterView):
         'name',
     ]
 
+    has_rows = True
+    model_row_class = model.Product
+
+    row_labels = {
+        'upc': "UPC",
+    }
+
+    row_grid_columns = [
+        'upc',
+        'brand',
+        'description',
+        'size',
+        'department',
+        'vendor',
+        'regular_price',
+        'current_price',
+    ]
+
     def configure_grid(self, g):
         super(ReportCodeView, self).configure_grid(g)
         g.filters['name'].default_active = True
@@ -58,6 +76,32 @@ class ReportCodeView(MasterView):
         g.set_link('code')
         g.set_link('name')
 
+    def get_row_data(self, reportcode):
+        return self.Session.query(model.Product)\
+                           .filter(model.Product.report_code == reportcode)
+
+    def get_parent(self, product):
+        return product.report_code
+
+    def configure_row_grid(self, g):
+        super(ReportCodeView, self).configure_row_grid(g)
+
+        app = self.get_rattail_app()
+        self.handler = app.get_products_handler()
+        g.set_renderer('regular_price', self.render_price)
+        g.set_renderer('current_price', self.render_price)
+
+        g.set_sort_defaults('upc')
+
+    def render_price(self, product, field):
+        if not product.not_for_sale:
+            price = product[field]
+            if price:
+                return self.handler.render_price(price)
+
+    def row_view_action_url(self, product, i):
+        return self.request.route_url('products.view', uuid=product.uuid)
+
 # TODO: deprecate / remove this
 ReportCodesView = ReportCodeView
 
