@@ -2,7 +2,7 @@
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2020 Lance Edgar
+#  Copyright © 2010-2021 Lance Edgar
 #
 #  This file is part of Rattail.
 #
@@ -29,7 +29,7 @@ from __future__ import unicode_literals, absolute_import
 from tailbone.views import MasterView
 
 
-class TablesView(MasterView):
+class TableView(MasterView):
     """
     Master view for tables
     """
@@ -70,6 +70,9 @@ class TablesView(MasterView):
         g.sorters['row_count'] = g.make_simple_sorter('row_count')
         g.set_sort_defaults('name')
 
+# TODO: deprecate / remove this
+TablesView = TableView
+
 
 def includeme(config):
-    TablesView.defaults(config)
+    TableView.defaults(config)
