@@ -2,7 +2,7 @@
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2021 Lance Edgar
+#  Copyright © 2010-2022 Lance Edgar
 #
 #  This file is part of Rattail.
 #
@@ -175,7 +175,7 @@ class VendorView(MasterView):
 
             # display
             {'section': 'rattail',
-             'option': 'vendor.use_autocomplete',
+             'option': 'vendors.choice_uses_dropdown',
              'type': bool},
         ]
 
